#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os, os.path
import subprocess

def add_cookie_to_xauthority(authority_file, display, cookie):
    command = ['xauth', '-f', authority_file, 'add', display, 'MIT-MAGIC-COOKIE-1', cookie]
    return exec_xauth(authority_file, command)

def remove_display_from_xauthority(authority_file, display):
    command = ['xauth', '-f', authority_file, 'remove', display]
    return exec_xauth(authority_file, command)

def exec_xauth(authority_file, command):
    statinfo = None
    # Backup the file attributes if the file exists
    if os.path.lexists(authority_file):
        statinfo = os.stat(authority_file)
    retcode = subprocess.call(command, stdout=None, stderr=None)
    # Revert to original owner
    if statinfo is not None:
        os.lchown(authority_file, statinfo.st_uid, statinfo.st_gid)
    if retcode == 0:
        return True
    return False

def merge_cookie_from_xauthority(source_file, dest_file, display):
    statinfo = None
    # Backup the file attributes if the file exists
    if os.path.lexists(dest_file):
        statinfo = os.stat(dest_file)
    cmd = 'xauth -f "%s" extract - %s | xauth -f "%s" merge -' % (source_file, display, dest_file)
    retcode = subprocess.call(cmd, shell=True, stdout=None, stderr=None)
    # Revert to original owner
    if statinfo is not None:
        os.lchown(dest_file, statinfo.st_uid, statinfo.st_gid)
    if retcode == 0:
        return True
    return False

def run_program(program, authority_file, display):
    environ = {}
    environ.update(os.environ)
    environ['DISPLAY'] = display
    environ['XAUTHORITY'] = authority_file
    ret = subprocess.call(program, shell=True, stdout=None, stderr=None, env=environ)
    return ret

